﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ServiceModel;

namespace TestApp
{
    public class ServiceSample : MarshalByRefObject, IDisposable
    {
        ServiceHost host;

        public ServiceSample()
        {
            host = new ServiceHost(typeof(OrderSvr));
        }

        public void RunService()
        {
            host.Open();
        }

        public void Dispose()
        {
            if (host != null)
            {
                if (host.State == CommunicationState.Opened)
                {
                    host.Close();
                }
            }
        }
    }

    [ServiceContract(Namespace = "demo-app")]
    public interface IOrder
    {
        [OperationContract]
        bool NewOrder(DateTime date, decimal price, long q);
    }

    //[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
    internal class OrderSvr : IOrder
    {
        public bool NewOrder(DateTime date, decimal price, long q)
        {
            // 验证
            if (date < DateTime.Now)
            {
                FaultReasonText text = new FaultReasonText("至少要在今天下单");
                FaultReason reason = new FaultReason(text);
                throw new FaultException(reason);
            }
            if (price < 0 || q < 0)
            {
                FaultReason reason = new FaultReason("单价或数量不能小于0");
                throw new FaultException(reason);
            }
            // 模拟下单
            if ((q * price) == 0.00M)
            {
                return false;
            }
            return true;
        }
    }

    // 这个方便调用
    public class SampleClient : ClientBase<IOrder>, IOrder
    {
        public SampleClient() : base("epcl") { }

        public bool NewOrder(DateTime date, decimal price, long q)
        {
            return Channel.NewOrder(date, price, q);
        }
    }
}
